# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondnametools
from abjad.tools import markuptools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import new
from abjad.tools.topleveltools import override


class TextSpanner(Spanner):
    r'''Text spanner.

    ..  container:: example

        A text spanner with no grob overrides:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> text_spanner = spannertools.TextSpanner()
            >>> attach(text_spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 \startTextSpan
                d'4
                e'4
                f'4 \stopTextSpan
            }

        This is (notationally unlikely) default behavior.

    ..  container:: example

        A text spanner with a grob override for left text:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> text_spanner = spannertools.TextSpanner()
            >>> markup = Markup('foo').italic().bold()
            >>> override(text_spanner).text_spanner.bound_details__left__text = markup
            >>> override(text_spanner).text_spanner.bound_details__left__stencil_align_dir_y = 0
            >>> attach(text_spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \override TextSpanner.bound-details.left.stencil-align-dir-y = #0
                \override TextSpanner.bound-details.left.text = \markup {
                    \bold
                        \italic
                            foo
                    }
                c'4 \startTextSpan
                d'4
                e'4
                f'4 \stopTextSpan
                \revert TextSpanner.bound-details
            }

    ..  container:: example

        Text spanner interacting with annotated markup. At the beginning of the
        spanner:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> markup = Markup('pont.')
            >>> attach(markup, staff[0], is_annotation=True)
            >>> text_spanner = spannertools.TextSpanner()
            >>> attach(text_spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 ^ \markup { pont. }
                d'4
                e'4
                f'4
            }

        Text spanner is suppresssed and only the markup appears.

    ..  container:: example

        Text spanner interacting with annotated markup. At the end of the
        spanner:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> markup = Markup('tasto')
            >>> attach(markup, staff[-1], is_annotation=True)
            >>> text_spanner = spannertools.TextSpanner()
            >>> attach(text_spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                d'4
                e'4
                f'4 ^ \markup { tasto }
            }

        Text spanner is suppresssed and only the markup appears.

    ..  container:: example

        Text spanner interacting with annotated markup. At the beginning and
        the end of the spanner:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> markup = Markup('pont.')
            >>> attach(markup, staff[0], is_annotation=True)
            >>> markup = Markup('tasto')
            >>> attach(markup, staff[-1], is_annotation=True)
            >>> text_spanner = spannertools.TextSpanner()
            >>> attach(text_spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 ^ \markup { pont. }
                d'4
                e'4
                f'4 ^ \markup { tasto }
            }

        Text spanner is suppresssed and only the markup appear.
        
    ..  container:: example

        Requires at least two leaves:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> text_spanner = spannertools.TextSpanner()
            >>> attach(text_spanner, staff[:1])
            Traceback (most recent call last):
                ...
            Exception: TextSpanner() attachment test fails for Selection([Note("c'4")]).

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_skip_attachment_test_all',
        )

    ### INITIALIZER ###

    def __init__(self, overrides=None):
        Spanner.__init__(self, overrides=overrides)
        self._skip_attachment_test_all = False

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, argument):
        if self._skip_attachment_test_all:
            return True
        return self._at_least_two_leaves(argument)

    def _get_annotations(self, leaf):
        inspector = inspect_(leaf)
        markups = []
        prototype = markuptools.Markup
        if inspector.has_indicator(prototype):
            indicator_expressions = inspector.get_indicators(
                markuptools.Markup,
                unwrap=False,
                )
            for indicator_expression in indicator_expressions:
                if indicator_expression.is_annotation:
                    markups.append(indicator_expression.indicator)
        markups = markups or None
        line_segment = None
        prototype = indicatortools.LineSegment
        if inspector.has_indicator(prototype):
            line_segment = inspector.get_indicator(prototype)
        return (
            markups,
            line_segment,
            )

    def _get_lilypond_format_bundle(self, component=None):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(
            component,
            )
        current_annotations = self._get_annotations(component)
        current_markups = current_annotations[0]
        current_markup = bool(current_markups)
        current_line_segment = current_annotations[1]
        start_spanner = self._spanner_starts_on_leaf(component)
        stop_spanner = self._spanner_stops_on_leaf(component)
        if start_spanner:
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            string = r'\startTextSpan'
            lilypond_format_bundle.right.spanner_starts.append(string)
        if stop_spanner:
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = r'\stopTextSpan'
            lilypond_format_bundle.right.spanner_stops.append(string)
        if current_markups is not None:
            # assign markup to spanner left text
            if start_spanner:
                markup = current_markups[0]
                if current_line_segment:
                    if current_line_segment.left_hspace is not None:
                        hspace = current_line_segment.left_hspace
                        hspace = markuptools.Markup.hspace(hspace)
                        markup = markuptools.Markup.concat([markup, hspace])
                override_ = lilypondnametools.LilyPondGrobOverride(
                    grob_name='TextSpanner',
                    is_once=True,
                    property_path=(
                        'bound-details',
                        'left',
                        'text',
                        ),
                    value=markup,
                    )
                override_string = override_.override_string
                lilypond_format_bundle.grob_overrides.append(override_string)
            # format markup normally
            else:
                current_markup = current_markups[0]
                markup = new(current_markup, direction=Up)
                string = format(markup, 'lilypond')
                lilypond_format_bundle.right.markup.append(string)
        if current_line_segment is not None:
            overrides = current_line_segment._get_lilypond_grob_overrides()
            for override_ in overrides:
                override_string = override_.override_string
                lilypond_format_bundle.grob_overrides.append(override_string)
        return lilypond_format_bundle

    def _get_previous_annotations(self, component):
        from abjad.tools import scoretools
        if not isinstance(component, scoretools.Leaf):
            return None, None
        leaves = self._get_leaves()
        index = leaves.index(component)
        for index in reversed(range(index)):
            previous_leaf = leaves[index]
            annotations = self._get_annotations(previous_leaf)
            if any(_ is not None for _ in annotations):
                return annotations
        return None, None

    def _leaf_has_current_event(self, leaf):
        annotations = self._get_annotations(leaf)
        markup = bool(annotations[0])
        line_segment = bool(annotations[1])
        return markup or line_segment

    def _leaf_has_markup(self, leaf):
        annotations = self._get_annotations(leaf)
        markup = bool(annotations[0])
        return markup

    def _spanner_has_smart_events(self):
        leaves = self._get_leaves()
        for leaf in leaves:
            if self._leaf_has_current_event(leaf):
                return True
        return False

    def _spanner_is_open_immediately_before_leaf(self, leaf):
        from abjad.tools import scoretools
        if not isinstance(leaf, scoretools.Leaf):
            return False
        leaves = self._get_leaves()
        index = leaves.index(leaf)
        for index in reversed(range(index)):
            previous_leaf = leaves[index]
            if self._spanner_starts_on_leaf(previous_leaf):
                return True
            if self._leaf_has_markup(previous_leaf):
                return False
        return False

    def _spanner_starts_on_leaf(self, leaf):
        annotations = self._get_annotations(leaf)
        markup = bool(annotations[0])
        line_segment = annotations[1]
        has_smart_events = self._spanner_has_smart_events()
        if not has_smart_events and self._is_my_first_leaf(leaf):
            return True
        if line_segment:
            return True
        return False

    def _spanner_stops_on_leaf(self, leaf):
        annotations = self._get_annotations(leaf)
        markup = bool(annotations[0])
        line_segment = annotations[1]
        spanner_is_open = self._spanner_is_open_immediately_before_leaf(leaf)
        if spanner_is_open and self._leaf_has_current_event(leaf):
            return True
        if spanner_is_open and self._is_my_last_leaf(leaf):
            return True
        return False



# -*- coding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools import markuptools
from abjad.tools.abctools import AbjadObject


class WoodwindFingering(AbjadObject):
    r'''A woodwind fingering.

    Initializes from a valid instrument name and up to three keyword
    lists or tuples:

    ::

        >>> center_column = ('one', 'two', 'three', 'five')
        >>> left_hand = ('R', 'thumb')
        >>> right_hand = ('e',)
        >>> woodwind_fingering = instrumenttools.WoodwindFingering(
        ...     instrument_name='clarinet',
        ...     center_column=center_column,
        ...     left_hand=left_hand,
        ...     right_hand=right_hand,
        ...     )

    ::

        >>> print(format(woodwind_fingering, 'storage'))
        instrumenttools.WoodwindFingering(
            instrument_name='clarinet',
            center_column=('one', 'two', 'three', 'five'),
            left_hand=('R', 'thumb'),
            right_hand=('e',),
            )

    Initializes a WoodwindFingering from another WoodwindFingering:

    ::

        >>> woodwind_fingering_2 = instrumenttools.WoodwindFingering(
        ...     woodwind_fingering)
        >>> print(format(woodwind_fingering_2))
        instrumenttools.WoodwindFingering(
            instrument_name='clarinet',
            center_column=('one', 'two', 'three', 'five'),
            left_hand=('R', 'thumb'),
            right_hand=('e',),
            )

    Calls a WoodwindFingering to create a woodwind diagram MarkupCommand:

    ::

        >>> fingering_command = woodwind_fingering()
        >>> print(format(fingering_command))
        markuptools.MarkupCommand(
            'woodwind-diagram',
            schemetools.Scheme(
                'clarinet',
                quoting="'",
                ),
            schemetools.Scheme(
                schemetools.SchemePair('cc', ('one', 'two', 'three', 'five')),
                schemetools.SchemePair('lh', ('R', 'thumb')),
                schemetools.SchemePair('rh', ('e',)),
                quoting="'",
                )
            )

    Attaches the MarkupCommand to score components, such as a chord
    representing a multiphonic sound:

    ::

        >>> markup = markuptools.Markup(contents=fingering_command, direction=Down)
        >>> chord = Chord("<ds' fs''>4")
        >>> attach(markup, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        <ds' fs''>4
            _ \markup {
                \woodwind-diagram
                    #'clarinet
                    #'((cc . (one two three five)) (lh . (R thumb)) (rh . (e)))
                }

    Initializes fingerings for eight different woodwind instruments:

    ::

        >>> instrument_names = [
        ...     'piccolo', 'flute', 'oboe', 'clarinet', 'bass-clarinet',
        ...     'saxophone', 'bassoon', 'contrabassoon',
        ...     ]
        >>> for name in instrument_names:
        ...    instrumenttools.WoodwindFingering(name)
        ...
        WoodwindFingering(instrument_name='piccolo', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='flute', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='oboe', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='clarinet', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='bass-clarinet', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='saxophone', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='bassoon', center_column=(), left_hand=(), right_hand=())
        WoodwindFingering(instrument_name='contrabassoon', center_column=(), left_hand=(), right_hand=())

    An override displays diagrams symbolically instead of graphically:

    ::

        >>> chord = Chord("<e' as' gqf''>1")
        >>> fingering = instrumenttools.WoodwindFingering(
        ...     'clarinet',
        ...     center_column=['one', 'two', 'three', 'four'],
        ...     left_hand=['R','cis'],
        ...     right_hand=['fis'])
        >>> diagram = fingering()
        >>> not_graphical = markuptools.MarkupCommand(
        ...     'override',
        ...     schemetools.SchemePair('graphical', False))
        >>> markup = markuptools.Markup(contents=
        ...     [not_graphical, diagram], direction=Down)
        >>> attach(markup, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        <e' as' gqf''>1
            _ \markup {
                \override
                    #'(graphical . #f)
                \woodwind-diagram
                    #'clarinet
                    #'((cc . (one two three four)) (lh . (R cis)) (rh . (fis)))
                }

    The thickness and size of diagrams can also be changed with overrides:

    ::

        >>> chord = Chord("<e' as' gqf''>1")
        >>> fingering = instrumenttools.WoodwindFingering(
        ...     'clarinet',
        ...     center_column=('one', 'two', 'three', 'four'),
        ...     left_hand=('R','cis'),
        ...     right_hand=('fis',),
        ...     )
        >>> diagram = fingering()
        >>> not_graphical = markuptools.MarkupCommand(
        ...     'override',
        ...     schemetools.SchemePair('graphical', False))
        >>> size = markuptools.MarkupCommand(
        ...     'override', schemetools.SchemePair('size', .5))
        >>> thickness = markuptools.MarkupCommand(
        ...     'override', schemetools.SchemePair('thickness', .4))
        >>> markup = markuptools.Markup(contents=
        ...     [not_graphical, size, thickness, diagram], direction=Down)
        >>> attach(markup, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        <e' as' gqf''>1
            _ \markup {
                \override
                    #'(graphical . #f)
                \override
                    #'(size . 0.5)
                \override
                    #'(thickness . 0.4)
                \woodwind-diagram
                    #'clarinet
                    #'((cc . (one two three four)) (lh . (R cis)) (rh . (fis)))
                }

    Inspired by Mike Solomon's LilyPond woodwind diagrams.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name=None,
        center_column=None,
        left_hand=None,
        right_hand=None,
        ):
        assert isinstance(center_column, (type(None), list, tuple))
        assert isinstance(left_hand, (type(None), list, tuple))
        assert isinstance(right_hand, (type(None), list, tuple))
        # initialize from a string and up to three lists
        instrument_name = instrument_name or 'flute'
        if isinstance(instrument_name, str):
            assert instrument_name in self._valid_instrument_names
            self._instrument_name = instrument_name
            if center_column is None:
                self._center_column = ()
            else:
                self._center_column = tuple(center_column)
            if left_hand is None:
                self._left_hand = ()
            else:
                self._left_hand = tuple(left_hand)
            if right_hand is None:
                self._right_hand = ()
            else:
                self._right_hand = tuple(right_hand)
        # initialize from WoodwindDiagram with up to three overriding lists
        elif isinstance(instrument_name, type(self)):
            self._instrument_name = instrument_name.instrument_name
            self._center_column = instrument_name.center_column
            self._left_hand = instrument_name.left_hand
            self._right_hand = instrument_name.right_hand
            if center_column is not None:
                self._center_column = tuple(center_column)
            if left_hand is not None:
                self._left_hand = tuple(left_hand)
            if right_hand is not None:
                self._right_hand = tuple(right_hand)

    ### SPECIAL METHODS ###

    def __call__(self):
        '''Calls woodwind fingering.

        Returns markup command.
        '''
        key_groups_as_scheme = []
        cc_scheme_pair = schemetools.SchemePair('cc', self._center_column)
        key_groups_as_scheme.append(cc_scheme_pair)
        lh_scheme_pair = schemetools.SchemePair('lh', self._left_hand)
        key_groups_as_scheme.append(lh_scheme_pair)
        rh_scheme_pair = schemetools.SchemePair('rh', self._right_hand)
        key_groups_as_scheme.append(rh_scheme_pair)
        key_groups_as_scheme = schemetools.Scheme(
            key_groups_as_scheme[:], quoting="'")
        instrument_as_scheme = schemetools.Scheme(
            self._instrument_name, quoting="'")
        return markuptools.MarkupCommand(
            'woodwind-diagram',
            instrument_as_scheme,
            key_groups_as_scheme,
            )

    def __format__(self, format_specification=''):
        r'''Formats woodwind fingering.

        Set `format_specification` to `''` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    ### PUBLIC METHODS ###

    def print_guide(self):
        r'''Print read-only string containing instrument's valid key strings,
        instrument diagram, and syntax explanation.

        Returns string.
        '''
        if self._instrument_name == 'clarinet':
            lines = [
                'list of valid key strings for clarinet:',
                '',
                'cc',
                'possibilities for one:',
                '(one oneT one1qT oneT1q one1q one1qT1h one1hT1q one1qT3q one3qT1q one1qTF oneFT1q one1hT oneT1h one1h one1hT3q one3qT1h one1hTF oneFT1h one3qT oneT3q one3q one3qTF oneFT3q oneFT oneF)',
                'possibilities for two:',
                '(two twoT two1qT twoT1q two1q two1qT1h two1hT1q two1qT3q two3qT1q two1qTF twoFT1q two1hT twoT1h two1h two1hT3q two3qT1h two1hTF twoFT1h two3qT twoT3q two3q two3qTF twoFT3q twoFT twoF)',
                'possibilities for three:',
                '(three threeT three1qT threeT1q three1q three1qT1h three1hT1q three1qT3q three3qT1q three1qTF threeFT1q three1hT threeT1h three1h three1hT3q three3qT1h three1hTF threeFT1h three3qT threeT3q three3q three3qTF threeFT3q threeFT threeF)',
                'possibilities for four:',
                '(four fourT four1qT fourT1q four1q four1qT1h four1hT1q four1qT3q four3qT1q four1qTF fourFT1q four1hT fourT1h four1h four1hT3q four3qT1h four1hTF fourFT1h four3qT fourT3q four3q four3qTF fourFT3q fourFT fourF)',
                'possibilities for five:',
                '(five fiveT five1qT fiveT1q five1q five1qT1h five1hT1q five1qT3q five3qT1q five1qTF fiveFT1q five1hT fiveT1h five1h five1hT3q five3qT1h five1hTF fiveFT1h five3qT fiveT3q five3q five3qTF fiveFT3q fiveFT fiveF)',
                'possibilities for six:',
                '(six sixT six1qT sixT1q six1q six1qT1h six1hT1q six1qT3q six3qT1q six1qTF sixFT1q six1hT sixT1h six1h six1hT3q six3qT1h six1hTF sixFT1h six3qT sixT3q six3q six3qTF sixFT3q sixFT sixF)',
                'possibilities for h:',
                '(h hT h1qT hT1q h1q h1qT1h h1hT1q h1qT3q h3qT1q h1qTF hFT1q h1hT hT1h h1h h1hT3q h3qT1h h1hTF hFT1h h3qT hT3q h3q h3qTF hFT3q hFT hF)',
                '',
                'lh',
                'possibilities for thumb:',
                '(thumb thumbT)',
                'possibilities for R:','(R RT)',
                'possibilities for a:',
                '(a aT)',
                'possibilities for gis:',
                '(gis gisT)',
                'possibilities for ees:',
                '(ees eesT)',
                'possibilities for cis:',
                '(cis cisT)',
                'possibilities for f:',
                '(f fT)',
                'possibilities for e:',
                '(e eT)',
                'possibilities for fis:','(fis fisT)',
                '',
                'rh',
                'possibilities for one:',
                '(one oneT)',
                'possibilities for two:',
                '(two twoT)',
                'possibilities for three:',
                '(three threeT)',
                'possibilities for four:',
                '(four fourT)',
                'possibilities for b:',
                '(b bT)',
                'possibilities for fis:',
                '(fis fisT)',
                'possibilities for gis:',
                '(gis gisT)',
                'possibilities for e:',
                '(e eT)',
                'possibilities for f:',
                '(f fT)',
                '',
                'diagram syntax',
                '',
                '   Lilypond woodwind diagram syntax divides an instrument into keyholes and keys.',
                '   Keyholes belong to the central column (cc) group.',
                '   Keys belong to either left-hand (lh) or right-hand (rh) groups.',
                "   In Abjad's diagrams, central column (cc) keyholes appear along a central dotted line.",
                '   Keys are grouped relative to the presence or absence of a dividing horizontal line:',
                '   If a horizontal line divides a side of the diagram,',
                '   keys above the line are left-hand keys (lh),',
                '   and those below are right-hand keys (rh).',
                '   If no horizontal line appears, all keys on that side of the diagram are left-hand keys (lh).',
                '   A key located along the central dotted line will be grouped',
                '   according to the playing hand of the nearest keyhole fingers.',
                '',
                '   To draw half- or quarter-covered keys, and to draw trills,',
                '   refer to the comprehensive list of possible key strings that precedes this explanation.',
                '',
                '',
                '             a  gis',
                '    R        |',
                '            one',
                '    thumb    h',
                '            two',
                '             |  ees',
                ' --------- three',
                '             |',
                '         one | cis',
                '         two |    f',
                '       three | e',
                '        four |    fis',
                '             |',
                '            four',
                '             |',
                '            five',
                '           b |',
                '            six',
                '      fis    |',
                '         gis |',
                '      e      |',
                '          f  |',
                '',
                '   clarinet',
                '   as modeled in LilyPond by Mike Solomon',
                '   diagram explanation and key string index above',
                ''
                ]
            for line in lines:
                print(line)

    ### PRIVATE PROPERTIES ###

    @property
    def _valid_instrument_names(self):
        return (
            'piccolo',
            'flute',
            'oboe',
            'clarinet',
            'bass-clarinet',
            'saxophone',
            'bassoon',
            'contrabassoon',
            )

     ### PUBLIC PROPERTIES ###

    @property
    def center_column(self):
        r'''Tuple of contents of key strings in center
        column key group:

    ::

        >>> woodwind_fingering.center_column
        ('one', 'two', 'three', 'five')

    Returns tuple.
        '''
        return self._center_column

    @property
    def instrument_name(self):
        r'''String of valid woodwind instrument name:

    ::

        >>> woodwind_fingering.instrument_name
        'clarinet'

    Returns string.
        '''
        return self._instrument_name

    @property
    def left_hand(self):
        r'''Tuple of contents of key strings in left
        hand key group:

    ::

        >>> woodwind_fingering.left_hand
        ('R', 'thumb')

    Returns tuple.
        '''
        return self._left_hand

    @property
    def right_hand(self):
        r'''Tuple of contents of key strings in right
        hand key group:

    ::

        >>> woodwind_fingering.right_hand
        ('e',)

    Returns tuple.
        '''
        return self._right_hand



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeContainer


class GraphvizTableRow(TreeContainer):
    r'''A Graphviz table row.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        children=None,
        name=None,
        ):
        TreeContainer.__init__(
            self,
            children=children,
            name=name,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of Graphviz table row.

        Returns string.
        '''
        result = []
        result.append('<TR>')
        for x in self:
            result.append('    ' + str(x))
        result.append('</TR>')
        result = '\n'.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizTableCell,
            graphtools.GraphvizTableVerticalRule,
            )
        return prototype



# -*- coding: utf-8 -*-
import os
from abjad import *


def test_systemtools_AbjadConfiguration___init___01():

    abjad_configuration = systemtools.AbjadConfiguration()

    assert os.path.exists(abjad_configuration.configuration_directory_path)
    assert os.path.exists(abjad_configuration.configuration_file_path)
    assert os.path.exists(abjad_configuration.abjad_output_directory)

    keys = [
        'abjad_output_directory',
        'accidental_spelling',
        'lilypond_path',
        'midi_player',
        'pdf_viewer',
        'text_editor',
        ]

    for key in keys:
        assert key in abjad_configuration



from abjad import *


def test_scoretools_FixedDurationTuplet_is_invisible_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 c'8 c'8")

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            c'8
            c'8
        }
        '''
        )

    tuplet.is_invisible = True

    assert format(tuplet) == stringtools.normalize(
        r'''
        \scaleDurations #'(2 . 3) {
            c'8
            c'8
            c'8
        }
        '''
        )

    tuplet.is_invisible = False

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            c'8
            c'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TypedList


class SelectionList(TypedList):
    r'''Selection list.

    ..  container:: example

        ::

            >>> selections = selectiontools.SelectionList()

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import selectiontools
        return selectiontools.Selection



# -*- coding: utf-8 -*-


def remove_powers_of_two(n):
    r'''Removes powers of 2 from the factors of positive integer `n`.

    ..  container:: example

        ::

            >>> for n in range(10, 100, 10):
            ...     print('\t%s\t%s' % (n, mathtools.remove_powers_of_two(n)))
            ... 
                10 5
                20 5
                30 15
                40 5
                50 25
                60 15
                70 35
                80 5
                90 45

    Returns positive integer.
    '''
    if not isinstance(n, int):
        raise TypeError
    if n <= 0:
        raise ValueError
    while n % 2 == 0:
        n //= 2
    return n



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import shutil
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript


class CleanScript(CommandlineScript):
    r'''Removes *.pyc, *.swp files and __pycache__ and tmp* directories
    recursively in a path.

    ..  shell::

        ajv clean --help

    '''

    ### CLASS VARIABLES ###

    alias = 'clean'
    short_description = (
        'Clean *.pyc, *.swp, .cache,  __pycache__ and tmp* '
        'files and folders from PATH.'
        )

    ### PRIVATE METHODS ###

    def _process_args(self, arguments):
        if (not arguments.cache and
            not arguments.pyc and
            not arguments.pycache and
            not arguments.swp and
            not arguments.tmp):
            arguments.cache = True,
            arguments.pyc = True
            arguments.pycache = True
            arguments.swp = True
            arguments.tmp = True
        print('Cleaning...')
        if arguments.pyc:
            print('\t*.pyc files')
        if arguments.swp:
            print('\t*.swp files')
        if arguments.cache:
            print('\t.cache directories')
        if arguments.pycache:
            print('\t__pycache__ directories')
        if arguments.tmp:
            print('\ttmp* directories')

        for root_directory, directory_names, file_names in os.walk(
            arguments.path):
            if '.svn' in directory_names:
                directory_names.remove('.svn')
            extensions = ()
            if arguments.pyc:
                extensions += ('.pyc',)
            if arguments.swp:
                extensions += ('.swp',)
            for file_name in file_names:
                if file_name.endswith(extensions):
                    file_path = os.path.join(
                        root_directory,
                        file_name,
                        )
                    os.remove(file_path)
            directories_to_remove = []
            for directory_name in directory_names:
                directory = os.path.join(
                    root_directory,
                    directory_name,
                    )
                should_remove = False
                if arguments.cache:
                    if directory_name == '.cache':
                        should_remove = True
                if arguments.pycache:
                    if directory_name == '__pycache__':
                        should_remove = True
                if arguments.tmp:
                    if directory_name.startswith('tmp'):
                        should_remove = True
                if not os.listdir(directory):
                    should_remove = True
                if should_remove:
                    shutil.rmtree(directory)
                    directories_to_remove.append(directory_name)
            for directory_name in directories_to_remove:
                directory_names.remove(directory_name)

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            'path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )
        parser.add_argument(
            '--cache',
            action='store_true',
            help='delete *.cache folders',
            )
        parser.add_argument(
            '--pyc',
            action='store_true',
            help='delete *.pyc files',
            )
        parser.add_argument(
            '--pycache',
            action='store_true',
            help='delete __pycache__ folders',
            )
        parser.add_argument(
            '--swp',
            action='store_true',
            help='delete Vim *.swp file',
            )
        parser.add_argument(
            '--tmp',
            action='store_true',
            help='delete tmp* folders',
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Beam_detach_01():
    r'''Detach length-one spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ]
            cs'8
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    detach(beam, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam_detach_02():
    r'''Detach length-four spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            cs'8
            d'8
            ef'8 ]
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    detach(beam, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_logical_voice_from_component_01():
    r'''Iterates only notes.
    '''

    container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container_1.is_simultaneous = True
    container_1[0].name = 'voice 1'
    container_1[1].name = 'voice 2'

    container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
    container_2.is_simultaneous = True
    container_2[0].name = 'voice 1'
    container_2[1].name = 'voice 2'

    staff = Staff([container_1, container_2])
    leaves = list(iterate(staff).by_leaf())

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    component = leaves[-1]
    notes = iterate(component).by_logical_voice_from_component(
        Note, reverse=True)
    notes = list(notes)

    voice_2_first_half = staff[0][1]
    voice_2_second_half = staff[1][1]

    assert notes[0] is voice_2_second_half[1]
    assert notes[1] is voice_2_second_half[0]
    assert notes[2] is voice_2_first_half[1]
    assert notes[3] is voice_2_first_half[0]


def test_agenttools_IterationAgent_by_logical_voice_from_component_02():
    r'''Iterates all components.
    '''

    container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container_1.is_simultaneous = True
    container_1[0].name = 'voice 1'
    container_1[1].name = 'voice 2'

    container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
    container_2.is_simultaneous = True
    container_2[0].name = 'voice 1'
    container_2[1].name = 'voice 2'

    staff = Staff([container_1, container_2])
    leaves = list(iterate(staff).by_leaf())

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    leaf = leaves[-1]
    components = iterate(leaf).by_logical_voice_from_component(reverse=True)
    components = list(components)

    r'''
    Note(c'', 8)
    Voice{2}
    Note(b', 8)
    Voice{2}
    Note(f', 8)
    Note(e', 8)
    '''

    assert components[0] is leaves[-1]
    assert components[1] is staff[1][1]
    assert components[2] is leaves[-2]
    assert components[3] is staff[0][1]
    assert components[4] is staff[0][1][1]
    assert components[5] is staff[0][1][0]


def test_agenttools_IterationAgent_by_logical_voice_from_component_03():

    container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container_1.is_simultaneous = True
    container_1[0].name = 'voice 1'
    container_1[1].name = 'voice 2'

    container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
    container_2.is_simultaneous = True
    container_2[0].name = 'voice 1'
    container_2[1].name = 'voice 2'

    staff = Staff([container_1, container_2])
    leaves = list(iterate(staff).by_leaf())

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    leaf = leaves[0]
    notes = iterate(leaf).by_logical_voice_from_component(Note)
    notes = list(notes)

    voice_1_first_half = staff[0][0]
    voice_1_second_half = staff[1][0]

    assert notes[0] is voice_1_first_half[0]
    assert notes[1] is voice_1_first_half[1]
    assert notes[2] is voice_1_second_half[0]
    assert notes[3] is voice_1_second_half[1]


def test_agenttools_IterationAgent_by_logical_voice_from_component_04():

    container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
    container_1.is_simultaneous = True
    container_1[0].name = 'voice 1'
    container_1[1].name = 'voice 2'

    container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
    container_2.is_simultaneous = True
    container_2[0].name = 'voice 1'
    container_2[1].name = 'voice 2'

    staff = Staff([container_1, container_2])
    leaves = list(iterate(staff).by_leaf())

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    leaf = leaves[0]
    components = iterate(leaf).by_logical_voice_from_component()
    components = list(components)

    r'''
    c'8
    Voice{2}
    d'8
    Voice{2}
    g'8
    a'8
    '''

    assert components[0] is leaves[0]
    assert components[1] is staff[0][0]
    assert components[2] is leaves[1]
    assert components[3] is staff[1][0]
    assert components[4] is staff[1][0][0]
    assert components[5] is staff[1][0][1]


# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def append_spacer_skips_to_underfull_measures(argument):
    r'''Append spacer skips to underfull measures in `argument`:

    ::

        >>> staff = Staff(Measure((3, 8), "c'8 d'8 e'8") * 3)
        >>> detach(TimeSignature, staff[1])
        (TimeSignature((3, 8)),)
        >>> new_time_signature = TimeSignature((4, 8))
        >>> attach(new_time_signature, staff[1])
        >>> detach(TimeSignature, staff[2])
        (TimeSignature((3, 8)),)
        >>> new_time_signature = TimeSignature((5, 8))
        >>> attach(new_time_signature, staff[2])
        >>> staff[1].is_underfull
        True
        >>> staff[2].is_underfull
        True

    ::

        >>> scoretools.append_spacer_skips_to_underfull_measures(staff)
        [Measure((4, 8), "c'8 d'8 e'8 s1 * 1/8"), Measure((5, 8), "c'8 d'8 e'8 s1 * 1/4")]

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 3/8
                c'8
                d'8
                e'8
            }
            {
                \time 4/8
                c'8
                d'8
                e'8
                s1 * 1/8
            }
            {
                \time 5/8
                c'8
                d'8
                e'8
                s1 * 1/4
            }
        }

    Returns measures treated.
    '''
    from abjad.tools import scoretools

    treated_measures = []
    for measure in iterate(argument).by_class(scoretools.Measure):
        if measure.is_underfull:
            scoretools.append_spacer_skip_to_underfull_measure(measure)
            treated_measures.append(measure)

    return treated_measures



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Component__get_sibling_01():
    '''Returns component when index is in range.
    '''

    staff = Staff("c' d' e' f'")
    assert staff[1]._get_sibling(2) is staff[3]
    assert staff[1]._get_sibling(1) is staff[2]
    assert staff[1]._get_sibling(0) is staff[1]
    assert staff[1]._get_sibling(-1) is staff[0]


def test_scoretools_Component__get_sibling_02():
    r'''Returns none when index is out of range.
    '''

    staff = Staff("c' d' e' f'")
    assert staff[1]._get_sibling(99) is None


def test_scoretools_Component__get_sibling_03():
    r'''Returns none when component has no parent.
    '''

    staff = Staff("c' d' e' f'")
    assert staff._get_sibling(1) is None



# -*- coding: utf-8 -*-


def timespan_2_happens_during_timespan_1(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `timespan_2` happens
    during `timespan_1`.

    ::

        >>> relation = timespantools.timespan_2_happens_during_timespan_1()
        >>> print(format(relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_1.start_offset <= timespan_2.start_offset'),
                    timespantools.Inequality('timespan_2.stop_offset <= timespan_1.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    ..  container:: example

        Evaluates whether timespan ``[7/8, 8/8)`` happens
        during timespan ``[1/2, 3/2)``:

        ::

            >>> timespan_1 = timespantools.Timespan(Offset(1, 2), Offset(3, 2))
            >>> timespan_2 = timespantools.Timespan(Offset(7, 8), Offset(8, 8))
            >>> timespantools.timespan_2_happens_during_timespan_1(
            ...     timespan_1=timespan_1,
            ...     timespan_2=timespan_2,
            ...     )
            True

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'timespan_1.start_offset <= timespan_2.start_offset',
        'timespan_2.stop_offset <= timespan_1.stop_offset',
        ])

    time_relation = timespantools.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Measure___setitem___01():

    measure = Measure((3, 4), "c' d' e'")
    measure[:2] = 'r8'

    assert measure.is_underfull
    assert pytest.raises(Exception, 'f(measure)')


def test_scoretools_Measure___setitem___02():

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure[:2] = 'r8'

    assert not measure.is_underfull
    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            r8
            e'4
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.selectiontools.Selection import Selection


class Lineage(Selection):
    r'''A selection of components in the lineage of a component.

    ::

        >>> score = Score()
        >>> score.append(Staff(r"""\new Voice = "Treble Voice" { c'4 }""",
        ... name='Treble Staff'))
        >>> score.append(Staff(r"""\new Voice = "Bass Voice" { b,4 }""",
        ... name='Bass Staff'))

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \context Staff = "Treble Staff" {
                \context Voice = "Treble Voice" {
                    c'4
                }
            }
            \context Staff = "Bass Staff" {
                \context Voice = "Bass Voice" {
                    b,4
                }
            }
        >>

    ::

        >>> for x in selectiontools.Lineage(score): x
        ...
        <Score<<2>>>
        <Staff-"Treble Staff"{1}>
        Voice("c'4")
        Note("c'4")
        <Staff-"Bass Staff"{1}>
        Voice('b,4')
        Note('b,4')

    ::

        >>> for x in selectiontools.Lineage(score['Bass Voice']): x
        ...
        <Score<<2>>>
        <Staff-"Bass Staff"{1}>
        Voice('b,4')
        Note('b,4')

    Returns lineage.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_component',
        )

    ### INITIALIZER ###

    def __init__(self, component=None):
        from abjad.tools import scoretools
        assert isinstance(component, (scoretools.Component, type(None)))
        music = []
        if component is not None:
            music.extend(reversed(component._get_parentage(include_self=False)))
            music.append(component)
            music.extend(component._get_descendants(include_self=False))
        Selection.__init__(self, music)
        self._component = component

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''The component from which the selection was derived.
        '''
        return self._component



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject
from abjad.tools.topleveltools.new import new


class Accelerando(AbjadValueObject):
    r'''Accelerando.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> score = Score([staff])
            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[0])

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                        ^ \markup {
                            \large
                                \upright
                                    accel.
                            }
                    d'4
                    e'4
                    f'4
                }
            >>

    Accelerandi format as LilyPond markup.

    Accelerandi are not followed by any type of dashed line or other spanner.

    Use accelerandi with a tempo spanner to generate dashed lines and other
    spanners.

    ..  todo:: add example tempo spanner example.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_markup',
        )

    ### INITIALIZER ###

    def __init__(self, markup=None):
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        # TODO: make default scope work
        #self._default_scope = scoretools.Score
        self._default_scope = None
        if markup is not None:
            assert isinstance(markup, markuptools.Markup)
        self._markup = markup

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of accelerando.

        ..  container:: example

            String representation of accelerando with default markup:

            ::

                >>> print(str(indicatortools.Accelerando()))
                \markup {
                    \large
                        \upright
                            accel.
                    }

        ..  container:: example

            String representation of accelerando with custom markup:

            ::

                >>> markup = Markup(r'\bold { \italic { accelerando } }')
                >>> accelerando = indicatortools.Accelerando(markup=markup)
                >>> print(str(accelerando))
                \markup {
                    \bold
                        {
                            \italic
                                {
                                    accelerando
                                }
                        }
                    }

        Returns string.
        '''
        return str(self._to_markup())

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self)

    @property
    def _default_markup(self):
        from abjad.tools import markuptools
        contents = r'\large \upright accel.'
        return markuptools.Markup(contents=contents)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        markup = self._to_markup()
        markup = new(markup, direction=Up)
        markup_format_pieces = markup._get_format_pieces()
        lilypond_format_bundle.right.markup.extend(markup_format_pieces)
        return lilypond_format_bundle

    def _to_markup(self):
        if self.markup is not None:
            return self.markup
        return self._default_markup

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of accelerando.

        ..  container:: example

            ::

                >>> accelerando = Accelerando()
                >>> accelerando.default_scope is None
                True

        .. todo:: Default scope should return score.

        Returns none (but should return score).
        '''
        return self._default_scope

    @property
    def markup(self):
        r'''Gets markup of accelerando.

        ..  container:: example

            ::

                >>> markup = Markup(r'\bold { \italic { accel. } }')
                >>> accelerando = indicatortools.Accelerando(markup=markup)
                >>> print(str(accelerando.markup))
                \markup {
                    \bold
                        {
                            \italic
                                {
                                    accel.
                                }
                        }
                    }

        Returns markup or none.
        '''
        return self._markup



# -*-coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import systemtools
from abjad.tools import timespantools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import select
from abjad.tools.topleveltools import set_
Selection = selectiontools.Selection


class Spanner(AbjadObject):
    '''Spanner.

    Any type of object that stretches horizontally and encompasses some number
    of score components.

    Examples include beams, slurs, hairpins and trills.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_components',
        '_contiguity_constraint',
        '_indicator_expressions',
        '_lilypond_grob_name_manager',
        '_lilypond_setting_name_manager',
        '_name',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        name=None,
        ):
        overrides = overrides or {}
        self._components = []
        self._contiguity_constraint = 'logical voice'
        self._apply_overrides(overrides)
        self._indicator_expressions = []
        self._lilypond_setting_name_manager = None
        if name is not None:
            name = str(name)
        self._name = name

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''Is true when spanner contains `argument`.
        Otherwise false.

        Returns true or false.
        '''
        for x in self._components:
            if x is argument:
                return True
        else:
            return False

    def __copy__(self, *arguments):
        r'''Copies spanner.

        Does not copy spanner components.

        Returns new spanner.
        '''
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_lilypond_grob_name_manager', None) is not None:
            new._lilypond_grob_name_manager = copy.copy(override(self))
        if getattr(self, '_lilypond_setting_name_manager', None) is not None:
            new._lilypond_setting_name_manager = copy.copy(set_(self))
        self._copy_keyword_args(new)
        new._name = self.name
        return new

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Returns component.
        '''
        if isinstance(argument, slice):
            components = self._components.__getitem__(argument)
            return selectiontools.Selection(components)
        return self._components.__getitem__(argument)

    def __getnewargs__(self):
        r'''Gets new arguments of spanner.

        Returns empty tuple.
        '''
        return ()

    def __getstate__(self):
        r'''Gets state of spanner.

        Returns dictionary.
        '''
        state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                state[slot] = getattr(self, slot, None)
        return state

    def __len__(self):
        r'''Gets number of components in spanner.

        Returns nonnegative integer.
        '''
        return len(self.components)

    def __lt__(self, argument):
        r'''Is true when spanner is less than `argument`. Otherwise false.

        Trivial comparison to allow doctests to work.

        Returns true or false.
        '''
        if not isinstance(argument, Spanner):
            raise TypeError
        return repr(self) < repr(argument)

    ### PRIVATE METHODS ###

    def _append(self, component):
        assert self._attachment_test(component), (repr(component), repr(self))
        if self._contiguity_constraint == 'logical voice':
            components = self[-1:] + [component]
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                components), repr(components)
        component._spanners.add(self)
        self._components.append(component)

    def _append_left(self, component):
        components = [component] + self[:1]
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            components)
        component._spanners.add(self)
        self._components.insert(0, component)

    def _apply_overrides(self, overrides):
        import abjad
        namespace = abjad.__dict__.copy()
        manager = override(self)
        for key, value in overrides.items():
            grob_name, attribute = key.split('__', 1)
            grob_manager = getattr(manager, grob_name)
            if isinstance(value, str):
                if 'markuptools' in value or 'schemetools' in value:
                    value = eval(value, namespace, namespace)
            setattr(grob_manager, attribute, value)

    def _at_least_two_leaves(self, component_expression):
        leaves = select(component_expression).by_leaf()
        return 1 < len(leaves)

    def _attach(self, components):
        from abjad.tools import selectiontools
        assert not self, repr(self)
        if isinstance(components, scoretools.Component):
            self._append(components)
        elif isinstance(components, (list, tuple, selectiontools.Selection)):
            self._extend(components)
        else:
            raise TypeError(components)

    def _attachment_test(self, component):
        from abjad.tools import scoretools
        return isinstance(component, scoretools.Leaf)

    def _attachment_test_all(self, component_expression):
        return True

    def _block_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._block_component(component)

    def _block_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.remove(self)

    def _constrain_contiguity(self):
        r'''Not composer-safe.
        '''
        self._contiguity_constraint = 'logical_voice'

    def _copy(self, components):
        r'''Returns copy of spanner with `components`.
        `components` must be an iterable of components already
        contained in spanner.
        '''
        my_components = self._components[:]
        self._components = []
        result = copy.copy(self)
        self._components = my_components
        for component in components:
            assert component in self
        for component in components:
            result._components.append(component)
        result._unblock_all_components()
        return result

    def _copy_keyword_args(self, new):
        pass

    def _detach(self):
        self._sever_all_components()

    def _extend(self, components):
        component_input = list(self[-1:])
        component_input.extend(components)
        if self._contiguity_constraint == 'logical voice':
            if not Selection._all_are_contiguous_components_in_same_logical_voice(
                component_input):
                message = 'must be contiguous components'
                message += ' in same logical voice: {!r}.'
                message = message.format(component_input)
                raise Exception(message)
        for component in components:
            self._append(component)

    def _extend_left(self, components):
        component_input = components + list(self[:1])
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            component_input)
        for component in reversed(components):
            self._append_left(component)

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.extend(getattr(self, '_reverts', []))
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.extend(getattr(self, '_overrides', []))
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        return result

    def _fracture(self, i, direction=None):
        r'''Fractures spanner at `direction` of component at index `i`.

        Valid values for `direction` are ``Left``, ``Right`` and ``None``.

        Set `direction=None` to fracture on both left and right sides.

        Returns tuple of original, left and right spanners.
        '''
        if i < 0:
            i = len(self) + i
        if direction == Left:
            return self._fracture_left(i)
        elif direction == Right:
            return self._fracture_right(i)
        elif direction is None:
            left = self._copy(self[:i])
            right = self._copy(self[i + 1:])
            center = self._copy(self[i:i + 1])
            self._block_all_components()
            return self, left, center, right
        else:
            message = 'direction {!r} must be left, right or none.'
            message = message.format(direction)
            raise ValueError(message)

    def _fracture_left(self, i):
        left = self._copy(self[:i])
        right = self._copy(self[i:])
        self._block_all_components()
        return self, left, right

    def _fracture_right(self, i):
        left = self._copy(self[:i + 1])
        right = self._copy(self[i + 1:])
        self._block_all_components()
        return self, left, right

    def _fuse_by_reference(self, spanner):
        result = self._copy(self[:])
        result._extend(spanner.components)
        self._block_all_components()
        spanner._block_all_components()
        return [(self, spanner, result)]

    def _get_basic_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
        return lilypond_format_bundle

    def _get_duration(self, in_seconds=False):
        return sum(
            component._get_duration(in_seconds=in_seconds)
            for component in self
            )

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if self._compact_summary == '':
            values = []
        else:
            values = [self._compact_summary]
        if 'overrides' in names and not self.overrides:
            names.remove('overrides')
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            repr_args_values=values,
            storage_format_kwargs_names=names,
            )

    def _get_indicators(self, prototype=None, unwrap=True):
        from abjad.tools import indicatortools
        prototype = prototype or (object,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        prototype_objects, prototype_classes = [], []
        for indicator_prototype in prototype:
            if isinstance(indicator_prototype, type):
                prototype_classes.append(indicator_prototype)
            else:
                prototype_objects.append(indicator_prototype)
        prototype_objects = tuple(prototype_objects)
        prototype_classes = tuple(prototype_classes)
        matching_indicators = []
        for indicator in self._indicator_expressions:
            if isinstance(indicator, prototype_classes):
                matching_indicators.append(indicator)
            elif any(indicator == x for x in prototype_objects):
                matching_indicators.append(indicator)
            elif isinstance(indicator, indicatortools.IndicatorExpression):
                if isinstance(indicator.indicator, prototype_classes):
                    matching_indicators.append(indicator)
                elif any(indicator.indicator == x for x in prototype_objects):
                    matching_indicators.append(indicator)
        if unwrap:
            matching_indicators = [x.indicator for x in matching_indicators]
        matching_indicators = tuple(matching_indicators)
        return matching_indicators

    def _get_leaves(self):
        result = []
        for component in self._components:
            for node in iterate(component).depth_first():
                if isinstance(node, scoretools.Leaf):
                    result.append(node)
        return result

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        lilypond_format_bundle.get('before').spanners.extend(
            self._format_before_leaf(leaf))
        lilypond_format_bundle.get('right').spanners.extend(
            self._format_right_of_leaf(leaf))
        lilypond_format_bundle.get('after').spanners.extend(
            self._format_after_leaf(leaf))
        return lilypond_format_bundle

    def _get_my_first_leaf(self):
        for leaf in iterate(self).by_leaf():
            return leaf

    def _get_my_last_leaf(self):
        for leaf in iterate(self).by_leaf(reverse=True):
            return leaf

    def _get_my_nth_leaf(self, n):
        from abjad.tools import scoretools
        if not isinstance(n, int):
            raise TypeError
        if 0 <= n:
            leaves = iterate(self).by_leaf()
            for leaf_index, leaf in enumerate(leaves):
                if leaf_index == n:
                    return leaf
        else:
            leaves = iterate(self).by_leaf(reverse=True)
            for leaf_index, leaf in enumerate(leaves):
                leaf_number = -leaf_index - 1
                if leaf_number == n:
                    return leaf
        raise IndexError

    def _get_timespan(self, in_seconds=False):
        from abjad.tools import durationtools
        if len(self):
            start_offset = self[0]._get_timespan(
                in_seconds=in_seconds)._start_offset
            stop_offset = self[-1]._get_timespan(
                in_seconds=in_seconds)._stop_offset
        else:
            start_offset = durationtools.Duration(0)
            stop_offset = durationtools.Duration(0)
        return timespantools.Timespan(
            start_offset=start_offset,
            stop_offset=stop_offset,
            )

    def _index(self, component):
        return self._components.index(component)

    def _insert(self, i, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)
        self._components.insert(i, component)

    def _is_exterior_leaf(self, leaf):
        r'''True if leaf is first or last in spanner.
        True if next leaf or previous leaf is none.
        Otherwise false.
        '''
        if self._is_my_first_leaf(leaf):
            return True
        elif self._is_my_last_leaf(leaf):
            return True
        elif not leaf._get_leaf(-1) and not leaf._get_leaf(1):
            return True
        else:
            return False

    def _is_interior_leaf(self, leaf):
        leaves = self._get_leaves()
        if leaf not in leaves:
            return False
        if len(leaves) < 3:
            return False
        leaf_count = len(leaves)
        first_index = 0
        last_index = leaf_count - 1
        leaf_index = leaves.index(leaf)
        if first_index < leaf_index < last_index:
            return True
        return False

    def _is_my_first(self, leaf, prototype):
        for component in iterate(self).by_class(prototype):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_first_leaf(self, leaf):
        try:
            first_leaf = self._get_my_nth_leaf(0)
            return leaf is first_leaf
        except IndexError:
            return False

    def _is_my_last(self, leaf, prototype):
        for component in iterate(self).by_class(
            prototype,
            reverse=True,
            ):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_last_leaf(self, leaf):
        try:
            last_leaf = self._get_my_nth_leaf(-1)
            return leaf is last_leaf
        except IndexError:
            return False

    def _is_my_only(self, leaf, prototype):
        i = None
        components = iterate(self).by_class(prototype)
        for i, component in enumerate(components):
            if 0 < i:
                return False
        return i == 0

    def _is_my_only_leaf(self, leaf):
        return self._is_my_first_leaf(leaf) and self._is_my_last_leaf(leaf)

    def _remove(self, component):
        r'''Not composer-safe.
        '''
        self._sever_component(component)

    def _remove_component(self, component):
        r'''Not composer-safe.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                self._components.pop(i)
                break
        else:
            message = 'component {!r} not in spanner components list.'
            raise ValueError(message.format(component))

    def _reverse_components(self):
        r'''Reverses order of spanner components.

        Not composer-safe because reversing the order of spanner components
        could scramble components of some other spanner.

        Call method only as part of a full component- and spanner-reversal
        routine.

        Spanner subclasses with mapping variables (like the 'durations' list
        attaching to durated complex beam spanners) should override this
        method to reverse mapping elements.
        '''
        self._components.reverse()

    def _sever_all_components(self):
        r'''Not composer-safe.
        '''
        for n in reversed(range(len(self))):
            component = self[n]
            self._sever_component(component)

    def _sever_component(self, component):
        r'''Not composer-safe.
        '''
        self._block_component(component)
        self._remove_component(component)

    def _start_offset_in_me(self, leaf):
        leaf_start_offset = leaf._get_timespan().start_offset
        self_start_offset = self._get_timespan().start_offset
        return leaf_start_offset - self_start_offset

    def _stop_offset_in_me(self, leaf):
        leaf_start_offset = self._start_offset_in_me(leaf)
        leaf_stop_offset = leaf_start_offset + leaf._get_duration()
        return leaf_stop_offset

    def _unblock_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._unblock_component(component)

    def _unblock_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)

    def _unconstrain_contiguity(self):
        r'''Not composer-safe.
        '''
        self._contiguity_constraint = None

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self)
        if not len_self:
            return ''
        elif 0 < len_self <= 8:
            return ', '.join([str(x) for x in self])
        else:
            left = ', '.join([str(x) for x in self[:2]])
            right = ', '.join([str(x) for x in self[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., ' % number_in_middle
            return left + middle + right

    @property
    def _duration_in_seconds(self):
        duration = durationtools.Duration(0)
        for leaf in self.leaves:
            duration += leaf._get_duration(in_seconds=True)
        return duration

    @property
    def _preprolated_duration(self):
        return sum([component._preprolated_duration for component in self])

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self])
        else:
            return ' '

    ### PUBLIC PROPERTIES ###

    @property
    def components(self):
        r'''Gets components in spanner.

        Returns selection.
        '''
        from abjad.tools import selectiontools
        return selectiontools.Selection(self._components[:])

    @property
    def name(self):
        r'''Gets spanner name.

        Returns string.
        '''
        return self._name

    @property
    def overrides(self):
        r'''Gets overrides.

        Returns dict.
        '''
        manager = override(self)
        overrides = {}
        for attribute_tuple in manager._get_attribute_tuples():
            attribute = '__'.join(attribute_tuple[:-1])
            value = attribute_tuple[-1]
            overrides[attribute] = value
        return overrides



# -*- coding: utf-8 -*-
import copy
from abjad.tools import graphtools
from abjad.tools import stringtools


def test_documentationtools_GraphvizGraph_deepcopy_01():
    graph = graphtools.GraphvizGraph()
    assert str(graph) == stringtools.normalize(
        r'''
        digraph G {
        }
        ''')
    copied = copy.deepcopy(graph)
    assert str(graph) == str(copied)


def test_documentationtools_GraphvizGraph_deepcopy_02():
    graph = graphtools.GraphvizGraph()
    graph.append(graphtools.GraphvizSubgraph())
    assert str(graph) == stringtools.normalize(
        r'''
        digraph G {
            subgraph cluster_0 {
            }
        }
        ''')
    copied = copy.deepcopy(graph)
    assert str(graph) == str(copied)


def test_documentationtools_GraphvizGraph_deepcopy_03():
    graph = graphtools.GraphvizGraph()
    graph.append(graphtools.GraphvizNode())
    graph.append(graphtools.GraphvizNode())
    assert str(graph) == stringtools.normalize(
        r'''
        digraph G {
            node_0;
            node_1;
        }
        ''')
    copied = copy.deepcopy(graph)
    assert str(graph) == str(copied)


def test_documentationtools_GraphvizGraph_deepcopy_04():
    graph = graphtools.GraphvizGraph()
    graph.append(graphtools.GraphvizNode())
    graph.append(graphtools.GraphvizNode())
    graphtools.GraphvizEdge().attach(graph[0], graph[1])
    assert str(graph) == stringtools.normalize(
        r'''
        digraph G {
            node_0;
            node_1;
            node_0 -> node_1;
        }
        ''')
    copied = copy.deepcopy(graph)
    assert str(graph) == str(copied)


def test_documentationtools_GraphvizGraph_deepcopy_05():
    graph = graphtools.GraphvizGraph()
    graph.append(graphtools.GraphvizSubgraph())
    graph[0].append(graphtools.GraphvizNode())
    graph[0].append(graphtools.GraphvizNode())
    graph[0].append(graphtools.GraphvizNode())
    graph[0].append(graphtools.GraphvizSubgraph())
    graph[0][-1].append(graphtools.GraphvizNode())
    graph.append(graphtools.GraphvizNode())
    graphtools.GraphvizEdge().attach(graph[0][1], graph[1])
    graphtools.GraphvizEdge().attach(graph[0][0], graph[0][-1][0])
    assert str(graph) == stringtools.normalize(
        r'''
        digraph G {
            subgraph cluster_0 {
                node_0_0;
                node_0_1;
                node_0_2;
                subgraph cluster_0_3 {
                    node_0_3_0;
                }
                node_0_0 -> node_0_3_0;
            }
            node_1;
            node_0_1 -> node_1;
        }
        '''
        )
    copied = copy.deepcopy(graph)
    assert str(graph) == str(copied)



# -*- coding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach


class PartCantusScoreTemplate(abctools.AbjadObject):
    r'''Pärt Cantus score template.
    '''

    ### SPECIAL METHODS ###

    def __call__(self):
        '''Calls score template.

        Returns LilyPond file.
        '''

        # make bell voice and staff
        bell_voice = scoretools.Voice(name='Bell Voice')
        bell_staff = scoretools.Staff([bell_voice], name='Bell Staff')
        clef = indicatortools.Clef('treble')
        attach(clef, bell_staff)
        bells = instrumenttools.Instrument(
            instrument_name='Campana in La',
            short_instrument_name='Camp.',
            pitch_range='[C4, C6]',
            )
        attach(bells, bell_staff)
        tempo = indicatortools.Tempo((1, 4), (112, 120))
        attach(tempo, bell_staff)
        time_signature = indicatortools.TimeSignature((6, 4))
        attach(time_signature, bell_staff)

        # make first violin voice and staff
        first_violin_voice = scoretools.Voice(name='First Violin Voice')
        first_violin_staff = scoretools.Staff(
            [first_violin_voice],
            name='First Violin Staff',
            )
        clef = indicatortools.Clef('treble')
        attach(clef, first_violin_staff)
        violin = instrumenttools.Violin(
            instrument_name_markup=markuptools.Markup('Violin I'),
            short_instrument_name_markup=markuptools.Markup('Vl. I'),
            )
        attach(violin, first_violin_staff)

        # make second violin voice and staff
        second_violin_voice = scoretools.Voice(name='Second Violin Voice')
        second_violin_staff = scoretools.Staff(
            [second_violin_voice],
            name='Second Violin Staff',
            )
        clef = indicatortools.Clef('treble')
        attach(clef, second_violin_staff)
        violin = instrumenttools.Violin(
            instrument_name_markup=markuptools.Markup('Violin II'),
            short_instrument_name_markup=markuptools.Markup('Vl. II'),
            )
        attach(violin, second_violin_staff)

        # make viola voice and staff
        viola_voice = scoretools.Voice(name='Viola Voice')
        viola_staff = scoretools.Staff([viola_voice], name='Viola Staff')
        clef = indicatortools.Clef('alto')
        attach(clef, viola_staff)
        viola = instrumenttools.Viola()
        attach(viola, viola_staff)

        # make cello voice and staff
        cello_voice = scoretools.Voice(name='Cello Voice')
        cello_staff = scoretools.Staff([cello_voice], name='Cello Staff')
        clef = indicatortools.Clef('bass')
        attach(clef, cello_staff)
        cello = instrumenttools.Cello()
        attach(cello, cello_staff)

        # make bass voice and staff
        bass_voice = scoretools.Voice(name='Bass Voice')
        bass_staff = scoretools.Staff([bass_voice], name='Bass Staff')
        clef = indicatortools.Clef('bass')
        attach(clef, bass_staff)
        contrabass = instrumenttools.Contrabass(
            short_instrument_name_markup=markuptools.Markup('Cb.'),
            )
        attach(contrabass, bass_staff)

        # make strings staff group
        strings_staff_group = scoretools.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            bass_staff,
            ],
            name='Strings Staff Group',
            )

        # make score
        score = scoretools.Score([
            bell_staff,
            strings_staff_group,
            ],
            name='Pärt Cantus Score'
            )

        # return Pärt Cantus score
        return score


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitchClassColorMap___slots___01():
    r'''Numbered pitch-class color maps are immutable.
    '''

    pitches = [[-8, 2, 10, 21], [0, 11, 32, 41], [15, 25, 42, 43]]
    colors = ['red', 'green', 'blue']
    numbered_pitch_class_color_map = \
        pitchtools.NumberedPitchClassColorMap(pitches, colors)

    statement = "numbered_pitch_class_color_map.foo = 'bar'"
    assert pytest.raises(Exception, statement)



# -*- coding: utf-8 -*-
import abc
import logging
import os
import pickle
import ply
import traceback
from abjad.tools.abctools.AbjadObject import AbjadObject


class Parser(AbjadObject):
    r'''Abstract base class for Abjad parsers.

    Rules objects for lexing and parsing must be defined by overriding the
    abstract properties `lexer_rules_object` and `parser_rules_object`.

    For most parsers these properties should simply return `self`.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_debug',
        '_lexer',
        '_logger',
        '_parser',
        )

    ### INITIALIZER ###

    def __init__(self, debug=False):

        self._debug = bool(debug)
        self._lexer = None
        self._parser = None

        if self.debug:
            logging.basicConfig(
                level=logging.DEBUG,
                filename=self.logger_path,
                filemode='w',
                format='%(filename)10s:%(lineno)8d:%(message)s'
                )
            self._logger = logging.getLogger()
        else:
            self._logger = logging.getLogger()
            self._logger.addHandler(logging.NullHandler())

        self._lexer = ply.lex.lex(
            debug=self.debug,
            debuglog=self.logger,
            object=self.lexer_rules_object
            )

        if self.pickle_path and not os.path.exists(self.pickle_path):
            try:
                directory_path, _ = os.path.split(self.pickle_path)
                if not os.path.exists(directory_path):
                    os.makedirs(directory_path)
                string = pickle.dumps(None)
                with open(self.pickle_path, 'wb') as file_pointer:
                    file_pointer.write(string)
            except (IOError, OSError):
                traceback.print_exc()

        self._parser = ply.yacc.yacc(
            debug=self.debug,
            debuglog=self.logger,
            module=self.parser_rules_object,
            outputdir=self.output_path,
            picklefile=self.pickle_path,
            )

    ### SPECIAL METHODS ###

    def __call__(self, input_string):
        r'''Parse `input_string` and return result.
        '''

        if hasattr(self, '_setup'):
            self._setup()

        if self.debug:
            result = self.parser.parse_debug(
                input_string,
                lexer=self.lexer,
                debug=self.logger)
        else:
            result = self.parser.parse(
                input_string,
                lexer=self.lexer)

        if hasattr(self, '_cleanup'):
            result = self._cleanup(result)

        return result

    ### PUBLIC METHODS ###

    def tokenize(self, input_string):
        r'''Tokenize `input string` and print results.
        '''
        self.lexer.input(input_string)
        for token in self.lexer:
            print(token)

    ### PUBLIC PROPERTIES ###

    @property
    def debug(self):
        r'''True if the parser runs in debugging mode.
        '''
        return self._debug

    @property
    def lexer(self):
        r'''The parser's PLY Lexer instance.
        '''
        return self._lexer

    @abc.abstractproperty
    def lexer_rules_object(self):
        r'''The object containing the parser's lexical rule definitions.
        '''
        raise NotImplementedError

    @property
    def logger(self):
        r'''The parser's Logger instance.
        '''
        return self._logger

    @property
    def logger_path(self):
        r'''The output path for the parser's logfile.
        '''
        import sys
        if self.output_path is None:
            return None
        file_name = 'parselog_{}_{}.txt'.format(
            type(self).__name__,
            '-'.join(str(x) for x in sys.version_info),
            )
        return os.path.join(self.output_path, file_name)

    @property
    def output_path(self):
        r'''The output path for files associated with the parser.
        '''
        from abjad import abjad_configuration
        configuration_directory_path = \
            abjad_configuration.configuration_directory_path
        output_path = os.path.join(
            configuration_directory_path,
            'parsers',
            )
        if not os.path.isdir(output_path):
            try:
                os.makedirs(output_path)
            except (IOError, OSError):
                return None
        return output_path

    @property
    def parser(self):
        r'''The parser's PLY LRParser instance.
        '''
        return self._parser

    @abc.abstractproperty
    def parser_rules_object(self):
        r'''The object containing the parser's syntactical rule definitions.
        '''
        raise NotImplementedError

    @property
    def pickle_path(self):
        r'''The output path for the parser's pickled parsing tables.
        '''
        import sys
        if self.output_path is None:
            return None
        file_name = 'parse_tables_{}_{}.pkl'.format(
            type(self).__name__,
            '-'.join(str(x) for x in sys.version_info),
            )
        return os.path.join(self.output_path, file_name)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension_chord_name_01():

    chord_suspension = tonalanalysistools.ChordSuspension(4, 3)
    assert chord_suspension.chord_name == 'sus4'

    chord_suspension = tonalanalysistools.ChordSuspension(('flat', 2), 1)
    assert chord_suspension.chord_name == 'susb2'

    chord_suspension = tonalanalysistools.ChordSuspension()
    assert chord_suspension.chord_name == ''



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie__add_or_remove_notes_to_achieve_written_duration_01():
    r'''Change trivial logical tie to nontrivial logical tie.
    '''

    staff = Staff("c'8 [ ]")
    logical_tie = inspect_(staff[0]).get_logical_tie()
    logical_tie._add_or_remove_notes_to_achieve_written_duration(Duration(5, 32))

    r'''
    \new Staff {
        c'8 ~ [
        c'32 ]
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ~ [
            c'32 ]
        }
        '''
        )


def test_selectiontools_LogicalTie__add_or_remove_notes_to_achieve_written_duration_02():
    r'''Change nontrivial logical tie to trivial logical tie.
    '''

    staff = Staff("c'8 ~ [ c'32 ]")
    logical_tie = inspect_(staff[0]).get_logical_tie()
    logical_tie._add_or_remove_notes_to_achieve_written_duration(Duration(1, 8))

    r'''
    \new Staff {
        c'8 [ ]
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ]
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.scoretools.Container import Container


class Cluster(Container):
    '''A cluster.

    ::

        >>> cluster = scoretools.Cluster("c'8 <d' g'>8 b'8")
        >>> show(cluster) # doctest: +SKIP

    ..  doctest::

        >>> print(format(cluster))
        \makeClusters {
            c'8
            <d' g'>8
            b'8
        }

    ::

        >>> cluster
        Cluster("c'8 <d' g'>8 b'8")

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        )

    _is_counttime_component = True

    ### INITIALIZER ###

    def __init__(self, music=None):
        Container.__init__(self, music)
        self.is_simultaneous = False

    ### PRIVATE METHODS ###

    def _format_open_brackets_slot(self, bundle):
        result = []
        contributor = ('self_brackets', 'open')
        if self.is_simultaneous:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        contributions = [r'\makeClusters {}'.format(brackets_open[0])]
        result.append([contributor, contributions])
        return tuple(result)

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()



# -*- coding: utf-8 -*-
import importlib
import inspect
import pytest
import abjad
from abjad.tools import abjadbooktools
from abjad.tools import documentationtools
from abjad.tools import metertools
from abjad.tools import systemtools
from abjad.tools import tonalanalysistools


ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.LilyPondOutputProxy,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___format___01(class_):
    r'''All concrete classes have a storage format.
    '''
    if (
        '_get_storage_format_specification' not in dir(class_) or
        '_get_format_specification' not in dir(class_)
        ):
        return
    if inspect.isabstract(class_):
        return
    instance = class_()
    instance_format = format(instance, 'storage')
    assert isinstance(instance_format, str)
    assert not instance_format == ''


ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.LilyPondOutputProxy,
    metertools.Meter,
    tonalanalysistools.RootedChordClass,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___format___02(class_):
    r'''All storage-formattable classes have evaluable storage format.
    '''
    if '_get_storage_format_specification' not in dir(class_):
        return
    if inspect.isabstract(class_):
        return
    environment = abjad.__dict__.copy()
    environment.update(abjad.demos.__dict__)
    environment['abjadbooktools'] = importlib.import_module(
        'abjad.tools.abjadbooktools')
    environment['abjad'] = abjad
    instance_one = class_()
    instance_one_format = format(instance_one, 'storage')
    assert isinstance(instance_one_format, str)
    assert instance_one_format != ''
    instance_two = eval(instance_one_format, environment)
    instance_two_format = format(instance_two, 'storage')
    assert instance_one_format == instance_two_format



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import select


class EvenRunRhythmMaker(RhythmMaker):
    r'''Even run rhythm-maker.

    ..  container:: example

        Makes even run of notes each equal in duration to ``1/d``
        with ``d`` equal to the denominator of each division on which
        the rhythm-maker is called:

        ::

            >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker()

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 4/8
                    {
                        c'8 [
                        c'8
                        c'8
                        c'8 ]
                    }
                }
                {
                    \time 3/4
                    {
                        c'4
                        c'4
                        c'4
                    }
                }
                {
                    \time 2/4
                    {
                        c'4
                        c'4
                    }
                }
            }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        '_exponent',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beam_specifier=None,
        duration_spelling_specifier=None,
        exponent=None,
        tie_specifier=None,
        tuplet_spelling_specifier=None,
        ):
        if exponent is not None:
            assert mathtools.is_nonnegative_integer(exponent)
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            tuplet_spelling_specifier=tuplet_spelling_specifier,
            )
        self._exponent = exponent

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls even-run rhythm-maker on `divisions`.

        ..  container:: example

            ::

                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker()
                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> result = rhythm_maker(divisions)
                >>> for selection in result:
                ...     selection
                Selection([Container("c'8 c'8 c'8 c'8")])
                Selection([Container("c'4 c'4 c'4")])
                Selection([Container("c'4 c'4")])

        Returns a list of selections. Each selection holds a single container
        filled with notes.
        '''
        superclass = super(EvenRunRhythmMaker, self)
        return superclass.__call__(
            divisions,
            rotation=rotation,
            )

    def __format__(self, format_specification=''):
        r'''Formats even run rhythm-maker.

        ..  container:: example

            ::

                >>> f(rhythm_maker)
                rhythmmakertools.EvenRunRhythmMaker()

        Set `format_specification` to `''` or `'storage'`.

        Returns string.
        '''
        superclass = super(EvenRunRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _make_container(self, division):
        duration_spelling_specifier = self._get_duration_spelling_specifier()
        forbidden_written_duration = \
            duration_spelling_specifier.forbidden_written_duration
        time_signature = indicatortools.TimeSignature(division)
        implied_prolation = time_signature.implied_prolation
        numerator, denominator = division.pair
        denominator = mathtools.greatest_power_of_two_less_equal(denominator)
        assert mathtools.is_positive_integer_power_of_two(denominator)
        exponent = self.exponent or 0
        denominator_multiplier = 2 ** exponent
        denominator *= denominator_multiplier
        unit_duration = durationtools.Duration(1, denominator)
        if forbidden_written_duration is not None:
            multiplier = 1
            while forbidden_written_duration <= unit_duration:
                unit_duration /= 2
                multiplier *= 2
            numerator *= multiplier
        numerator *= denominator_multiplier
        notes = scoretools.make_notes(numerator * [0], [unit_duration])
        if implied_prolation == 1:
            result = scoretools.Container(notes)
        else:
            multiplier = implied_prolation
            result = scoretools.Tuplet(multiplier, notes)
        return result

    def _make_music(self, divisions, rotation):
        selections = []
        for division in divisions:
            prototype = mathtools.NonreducedFraction
            assert isinstance(division, prototype), division
        for division in divisions:
            container = self._make_container(division)
            selection = selectiontools.Selection(container)
            selections.append(selection)
        beam_specifier = self._get_beam_specifier()
        if beam_specifier.beam_divisions_together:
            durations = []
            for selection in selections:
                duration = selection.get_duration()
                durations.append(duration)
            beam = spannertools.DuratedComplexBeam(
                durations=durations,
                span_beam_count=1,
                nibs_towards_nonbeamable_components=False,
                )
            components = []
            for selection in selections:
                components.extend(selection)
            leaves = select(components).by_leaf()
            #attach(beam, components)
            attach(beam, leaves)
        elif beam_specifier.beam_each_division:
            for selection in selections:
                beam = spannertools.MultipartBeam()
                leaves = select(selection).by_leaf()
                #attach(beam, selection)
                attach(beam, leaves)
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier(
                ...     forbidden_written_duration=Duration(1, 4),
                ...     )
                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     duration_spelling_specifier=specifier,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                }

        Returns duration spelling specifier or none.
        '''
        return self._duration_spelling_specifier

    @property
    def exponent(self):
        r'''Gets exponent.

        ..  container:: example

            Makes even run of notes with durations equal to ``1/(2**0 * d)`` or
            ``1/d``:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     exponent=0,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'4
                            c'4
                            c'4
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'4
                            c'4
                        }
                    }
                }

            (``d`` equals the denominator of each input division.)

            Input division denominators ``8``, ``4``, ``4`` result in notes
            with durations ``1/8``, ``1/4``, ``1/4``.

        ..  container:: example

            Makes even run of notes with durations equal to ``1/(2**1 * d)`` or
            ``1/(2d)``:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     exponent=1,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                }

            (``d`` equals the denominator of each input division.)

            Input division denominators ``8``, ``4``, ``4`` result in notes
            with durations ``1/16``, ``1/8``, ``1/8``.

        ..  container:: example

            Makes even run of notes with durations equal to ``1/(2**2 * d)`` or
            ``1/(4d)``:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     exponent=2,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 4), (2, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'32 [
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 2/4
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

            (``d`` equals the denominator of each input division.)

            Input division denominators ``8``, ``4``, ``4`` result in notes
            with durations ``1/32``, ``1/16``, ``1/16``.

        Defaults to none.

        Interprets none equal to ``0``.

        Returns nonnegative integer or none.
        '''
        return self._exponent

    @property
    def tie_specifier(self):
        r'''Gets tie specifier.

        ..  container:: example

            Do not tie across any divisions:

            ::

                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=False,
                ...     )
                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (3, 8), (4, 8), (2, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 2/8
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Ties across all divisions:

            ::

                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=True,
                ...     )
                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (3, 8), (4, 8), (2, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 2/8
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Ties across every other pair of divisions (starting with the
            first):

            ::

                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=[1, 0],
                ...     )
                >>> rhythm_maker = rhythmmakertools.EvenRunRhythmMaker(
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (3, 8), (4, 8), (2, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 2/8
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }

        Returns true or false.
        '''
        return RhythmMaker.tie_specifier.fget(self)

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        ..  note:: not yet implemented.

        Returns tuplet spelling specifier or none.
        '''
        superclass = super(EvenRunRhythmMaker, self)
        return superclass.tuplet_spelling_specifier



# -*- coding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import iterate


class TonalAnalysisAgent(abctools.AbjadObject):
    r'''A tonal analysis interface.

    ..  container:: example

        **Example 1.** Interface to conjunct selection:

            >>> staff = Staff("c'4 d' e' f'")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }

        ::

            >>> selection_1 = tonalanalysistools.select(staff[:])

    ..  container:: example

        **Example 2.** Interface to disjunct selection:

            >>> staff = Staff("c'4 d' e' f'")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }

        ::

            >>> selection_2 = tonalanalysistools.select(staff[:1] + staff[-1:])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        from abjad.tools import selectiontools
        if not isinstance(client, selectiontools.Selection):
            client = selectiontools.Selection(client)
        self._client = client

    ### PRIVATE METHODS ###

    @staticmethod
    def _analyze_chord(argument):
        from abjad.tools import tonalanalysistools
        pitches = pitchtools.PitchSegment.from_selection(argument)
        npcset = pitchtools.PitchClassSet(
            pitches,
            item_class=pitchtools.NamedPitchClass,
            )
        ordered_npcs = []
        letters = ('c', 'e', 'g', 'b', 'd', 'f', 'a')
        for letter in letters:
            for npc in npcset:
                if npc.diatonic_pitch_class_name == letter:
                    ordered_npcs.append(npc)
        ordered_npcs = pitchtools.PitchClassSegment(
            ordered_npcs, item_class=pitchtools.NamedPitchClass)
        for x in range(len(ordered_npcs)):
            ordered_npcs = ordered_npcs.rotate(1)
            segment = pitchtools.IntervalClassSegment(
                    items=mathtools.difference_series(list(ordered_npcs)),
                    item_class=pitchtools.NamedInversionEquivalentIntervalClass,
                    )
            if segment.is_tertian:
                break
        else:
            return None
        root = ordered_npcs[0]
        class_ = tonalanalysistools.RootlessChordClass
        rootless_chord_class = class_.from_interval_class_segment(segment)
        bass = min(pitches).named_pitch_class
        inversion = ordered_npcs.index(bass)
        return tonalanalysistools.RootedChordClass(
            root,
            rootless_chord_class.quality_string,
            rootless_chord_class.extent,
            inversion,
            )

    @staticmethod
    def _analyze_incomplete_chord(argument):
        from abjad.tools import tonalanalysistools
        pitches = pitchtools.PitchSegment.from_selection(argument)
        npcset = pitchtools.PitchClassSet(
            pitches, item_class=pitchtools.NamedPitchClass)
        dicv = pitchtools.IntervalClassVector(
            items=npcset,
            item_class=pitchtools.NamedInversionEquivalentIntervalClass,
            )
        # TODO: eliminate code duplication #
        if dicv == TonalAnalysisAgent._make_dicv('c', 'ef'):
            model_npcs = ['c', 'ef']
            quality, extent = 'minor', 'triad'
        elif dicv == TonalAnalysisAgent._make_dicv('c', 'e'):
            model_npcs = ['c', 'e']
            quality, extent = 'major', 'triad'
        elif dicv == TonalAnalysisAgent._make_dicv('c', 'ef', 'bff'):
            model_npcs = ['c', 'ef', 'bff']
            quality, extent = 'diminished', 'seventh'
        elif dicv == TonalAnalysisAgent._make_dicv('c', 'ef', 'bf'):
            model_npcs = ['c', 'ef', 'bf']
            quality, extent = 'minor', 'seventh'
        elif dicv == TonalAnalysisAgent._make_dicv('c', 'e', 'bf'):
            model_npcs = ['c', 'e', 'bf']
            quality, extent = 'dominant', 'seventh'
        elif dicv == TonalAnalysisAgent._make_dicv('c', 'e', 'b'):
            model_npcs = ['c', 'e', 'b']
            quality, extent = 'major', 'seventh'
        else:
            message = 'can not identify incomplete tertian chord.'
            raise ValueError(message)
        bass = min(pitches).named_pitch_class
        try:
            npcseg = npcset.order_by(
                pitchtools.PitchClassSegment(
                    model_npcs,
                    item_class=pitchtools.NamedPitchClass,
                    ))
        except ValueError:
            message = 'can not identify incomplete tertian chord.'
            raise ValueError(message)
        inversion = npcseg.index(bass)
        root = npcseg[0]
        return tonalanalysistools.RootedChordClass(
            root,
            quality,
            extent,
            inversion,
            )

    @staticmethod
    def _analyze_incomplete_tonal_function(argument, key_signature):
        from abjad.tools import tonalanalysistools
        if isinstance(argument, tonalanalysistools.RootedChordClass):
            chord_class = argument
        else:
            selection = tonalanalysistools.select(argument)
            chord_classes = selection.analyze_incomplete_chords()
            assert len(chord_classes) == 1
            chord_class = chord_classes[0]
        root = chord_class.root
        scale = tonalanalysistools.Scale(key_signature)
        scale_degree = scale.named_pitch_class_to_scale_degree(root)
        quality = chord_class.chord_quality.quality_string
        extent = chord_class.extent
        inversion = chord_class.inversion
        return tonalanalysistools.RomanNumeral(
            scale_degree,
            quality,
            extent,
            inversion,
            )

    @staticmethod
    def _analyze_tonal_function(argument, key_signature):
        from abjad.tools import tonalanalysistools
        if isinstance(argument, tonalanalysistools.RootedChordClass):
            chord_class = argument
        else:
            selection = tonalanalysistools.select(argument)
            chord_classes = selection.analyze_chords()
            assert len(chord_classes) == 1
            chord_class = chord_classes[0]
        if chord_class is None:
            return None
        root = chord_class.root
        scale = tonalanalysistools.Scale(key_signature)
        scale_degree = scale.named_pitch_class_to_scale_degree(root)
        quality = chord_class.chord_quality.quality_string
        extent = chord_class.extent
        inversion = chord_class.inversion
        return tonalanalysistools.RomanNumeral(
            scale_degree,
            quality,
            extent,
            inversion,
            )

    @staticmethod
    def _is_neighbor_note(note):
        from abjad.tools import scoretools
        from abjad.tools import tonalanalysistools
        if not isinstance(note, scoretools.Note):
            message = 'must be note: {!r}.'
            message = message.format(note)
            raise TypeError(message)
        previous_note = note._get_in_my_logical_voice(
            -1, prototype=scoretools.Note)
        next_note = note._get_in_my_logical_voice(
            1, prototype=scoretools.Note)
        if previous_note is None:
            return False
        if next_note is None:
            return False
        notes = [previous_note, note, next_note]
        selection = tonalanalysistools.select(notes)
        preceding_interval = note.written_pitch - previous_note.written_pitch
        preceding_interval_direction = \
            mathtools.sign(preceding_interval.direction_number)
        following_interval = next_note.written_pitch - note.written_pitch
        following_interval_direction = \
            mathtools.sign(following_interval.direction_number)
        if selection.are_stepwise_notes():
            if preceding_interval_direction != following_interval_direction:
                return True
        return False

    @staticmethod
    def _is_passing_tone(note):
        from abjad.tools import scoretools
        from abjad.tools import tonalanalysistools
        if not isinstance(note, scoretools.Note):
            message = 'must be note: {!r}.'
            message = message.format(note)
            raise TypeError(message)
        previous_note = note._get_in_my_logical_voice(
            -1, prototype=scoretools.Note)
        next_note = note._get_in_my_logical_voice(
            1, prototype=scoretools.Note)
        if previous_note is None or next_note is None:
            return False
        notes = [previous_note, note, next_note]
        selection = tonalanalysistools.select(notes)
        return selection.are_scalar_notes()

    @staticmethod
    def _make_dicv(*named_pitch_classes):
        pitch_set = pitchtools.PitchSet(named_pitch_classes)
        return pitchtools.IntervalClassVector(
            items=pitch_set,
            item_class=pitchtools.NamedInversionEquivalentIntervalClass,
            )

    ### PUBLIC METHODS ###

    def analyze_chords(self):
        r'''Analyzes chords in selection.

        ..  container:: example

            ::

                >>> chord = Chord([7, 10, 12, 16], (1, 4))
                >>> tonalanalysistools.select(chord).analyze_chords()
                [CDominantSeventhInSecondInversion]

        Returns none when no tonal chord is understood.

        Returns list with elements each equal to chord class or none.
        '''
        result = []
        for component in self._client:
            chord_class = self._analyze_chord(component)
            result.append(chord_class)
        return result

    def analyze_incomplete_chords(self):
        r'''Analyzes incomplete chords in selection.

        ..  container:: example

            ::

                >>> chord = Chord("<g' b'>4")
                >>> tonalanalysistools.select(chord).analyze_incomplete_chords()
                [GMajorTriadInRootPosition]

        ..  container:: example

            ::

                >>> chord = Chord("<fs g b>4")
                >>> tonalanalysistools.select(chord).analyze_incomplete_chords()
                [GMajorSeventhInSecondInversion]

        Raises tonal harmony error when chord in selection can not analyze.

        Returns list with elements each equal to chord class or none.
        '''
        result = []
        for component in self._client:
            chord_class = self._analyze_incomplete_chord(component)
            result.append(chord_class)
        return result

    def analyze_incomplete_tonal_functions(self, key_signature):
        r'''Analyzes incomplete tonal functions of chords in selection
        according to `key_signature`.

        ..  container:: example

            ::

                >>> chord = Chord("<c' e'>4")
                >>> key_signature = KeySignature('g', 'major')
                >>> selection = tonalanalysistools.select(chord)
                >>> selection.analyze_incomplete_tonal_functions(key_signature)
                [IVMajorTriadInRootPosition]

        Raises tonal harmony error when chord in selection can not analyze.

        Returns list with elements each equal to tonal function or none.
        '''
        result = []
        for component in self._client:
            tonal_function = self._analyze_incomplete_tonal_function(
                component, key_signature)
            result.append(tonal_function)
        return result

    def analyze_neighbor_notes(self):
        r'''Is true when `note` in selection is preceeded by a stepwise interval
        in one direction and followed by a stepwise interval in
        the other direction. Otherwise false.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> selection = tonalanalysistools.select(staff[:])
                >>> selection.analyze_neighbor_notes()
                [False, False, False, False]

        Returns list of boolean values.
        '''
        result = []
        for component in self._client:
            tonal_function = self._is_neighbor_note(component)
            result.append(tonal_function)
        return result

    def analyze_passing_tones(self):
        r'''Is true when note in selection is both preceeded and followed
        by scalewise notes. Otherwise false.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> selection = tonalanalysistools.select(staff[:])
                >>> selection.analyze_passing_tones()
                [False, True, True, False]

        Returns list of boolean values.
        '''
        result = []
        for component in self._client:
            tonal_function = self._is_passing_tone(component)
            result.append(tonal_function)
        return result

    def analyze_tonal_functions(self, key_signature):
        r'''Analyzes tonal function of chords in selection
        according to `key_signature`.

        ..  container:: example

                >>> chord = Chord('<ef g bf>4')
                >>> key_signature = KeySignature('c', 'major')
                >>> selection = tonalanalysistools.select(chord)
                >>> selection.analyze_tonal_functions(key_signature)
                [FlatIIIMajorTriadInRootPosition]

        Returns none when no tonal function is understood.

        Returns list with elements each equal to tonal function or none.
        '''
        result = []
        for component in self._client:
            tonal_function = self._analyze_tonal_function(
                component, key_signature)
            result.append(tonal_function)
        return result

    def are_scalar_notes(self):
        r'''Is true when notes in selection are scalar.

        ::

            >>> selection_1.are_scalar_notes()
            True

        Otherwise false:

        ::


            >>> selection_2.are_scalar_notes()
            False

        Returns true or false.
        '''
        from abjad.tools import scoretools
        direction_string = None
        notes = iterate(self._client).by_class(scoretools.Note)
        for left, right in sequencetools.Sequence(notes).nwise():
            try:
                assert not (left.written_pitch == right.written_pitch)
                mdi = pitchtools.NamedInterval.from_pitch_carriers(
                    left, right)
                assert mdi.number <= 2
                if direction_string is None:
                    direction_string = mdi.direction_string
                assert direction_string == mdi.direction_string
            except AssertionError:
                return False
        return True

    def are_stepwise_ascending_notes(self):
        r'''Is true when notes in selection are stepwise ascending.

        ::

            >>> selection_1.are_stepwise_ascending_notes()
            True

        Otherwise false:

        ::

            >>> selection_2.are_stepwise_ascending_notes()
            False

        Returns true or false.
        '''
        from abjad.tools import scoretools
        notes = iterate(self._client).by_class(scoretools.Note)
        for left, right in sequencetools.Sequence(notes).nwise():
            try:
                assert not (left.written_pitch == right.written_pitch)
                mdi = pitchtools.NamedInterval.from_pitch_carriers(
                    left, right)
                assert mdi.number == 2
            except AssertionError:
                return False
        return True

    def are_stepwise_descending_notes(self):
        r'''Is true when notes in selection are stepwise descending.

        ::

            >>> selection_3 = tonalanalysistools.select(reversed(staff[:]))

        ::

            >>> selection_3.are_stepwise_descending_notes()
            True

        Otherwise false:

        ::

            >>> selection_1.are_stepwise_descending_notes()
            False

        ::

            >>> selection_2.are_stepwise_descending_notes()
            False

        Returns true or false.
        '''
        from abjad.tools import scoretools
        notes = iterate(self._client).by_class(scoretools.Note)
        for left, right in sequencetools.Sequence(notes).nwise():
            try:
                assert not (left.written_pitch == right.written_pitch)
                mdi = pitchtools.NamedInterval.from_pitch_carriers(
                    left, right)
                assert mdi.number == -2
            except AssertionError:
                return False
        return True

    def are_stepwise_notes(self):
        r'''Is true when notes in selection are stepwise.

        ::


            >>> selection_1.are_stepwise_notes()
            True

        Otherwise false:

        ::

            >>> selection_2.are_stepwise_notes()
            False

        Returns true or false.
        '''
        from abjad.tools import scoretools
        notes = iterate(self._client).by_class(scoretools.Note)
        for left, right in sequencetools.Sequence(notes).nwise():
            try:
                assert not (left.written_pitch == right.written_pitch)
                hdi = pitchtools.NamedInterval.from_pitch_carriers(
                    left, right)
                assert hdi.number <= 2
            except AssertionError:
                return False
        return True

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''Returns client of mutation agent.

        Returns selection or component.
        '''
        return self._client



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_empty_piano_score_01():

    score, treble, bass = scoretools.make_empty_piano_score()

    r'''
    \new Score <<
        \new PianoStaff <<
            \context Staff = "treble" {
                \clef "treble"
            }
            \context Staff = "bass" {
                \clef "bass"
            }
        >>
    >>
    '''

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                }
                \context Staff = "bass" {
                    \clef "bass"
                }
            >>
        >>
        '''
        )


# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_IncisedRhythmMaker___call___01():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-8],
        prefix_counts=[0, 1],
        suffix_talea=[-1],
        suffix_counts=[1],
        talea_denominator=32,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'2 ~
                c'16.
                r32
            }
            {
                r4
                c'4 ~
                c'16.
                r32
            }
            {
                c'2 ~
                c'16.
                r32
            }
            {
                r4
                c'4 ~
                c'16.
                r32
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___02():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-8],
        prefix_counts=[1, 2, 3, 4],
        suffix_talea=[-1],
        suffix_counts=[1],
        talea_denominator=32,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r4
                c'4 ~
                c'16.
                r32
            }
            {
                r4
                r4
                c'16.
                r32
            }
            {
                r4
                r4
                r8
            }
            {
                r4
                r4
                r8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___03():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[1],
        suffix_talea=[-8],
        suffix_counts=[1, 2, 3],
        talea_denominator=32,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r32
                c'4 ~
                c'16.
                r4
            }
            {
                r32
                c'16.
                r4
                r4
            }
            {
                r32
                r4
                r4
                r16.
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___04():

    incise_specifier = rhythmmakertools.InciseSpecifier()

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'2 ~
                c'8
            }
            {
                c'2 ~
                c'8
            }
            {
                c'2 ~
                c'8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___05():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[1],
        suffix_talea=[-1],
        suffix_counts=[1],
        talea_denominator=8,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    r8
                    c'4.
                    r8
                }
            }
            {
                {
                    r8
                    c'4
                    r8
                }
            }
            {
                \times 4/7 {
                    r8
                    c'2 ~
                    c'8
                    r8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___06():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[1],
        talea_denominator=32,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[2, 0],
        split_divisions_by_counts=[20],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 8/9 {
                    r32
                    c'2 ~
                    c'32
                }
            }
            {
                {
                    r32
                    c'16.
                }
                \tweak text #tuplet-number::calc-fraction-text
                \times 6/7 {
                    r32
                    c'4. ~
                    c'32
                }
            }
            {
                {
                    r32
                    c'8..
                }
                \times 4/5 {
                    r32
                    c'4 ~
                    c'32
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___07():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[8],
        prefix_counts=[0, 1],
        suffix_talea=[1],
        suffix_counts=[1],
        talea_denominator=32,
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r2
                r16.
                c'32
            }
            {
                c'4
                r4
                r16.
                c'32
            }
            {
                r2
                r16.
                c'32
            }
            {
                c'4
                r4
                r16.
                c'32
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___08():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[8],
        prefix_counts=[1, 2, 3, 4],
        suffix_talea=[1],
        suffix_counts=[1],
        talea_denominator=32,
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'4
                r4
                r16.
                c'32
            }
            {
                c'4
                c'4
                r16.
                c'32
            }
            {
                c'4
                c'4
                c'8
            }
            {
                c'4
                c'4
                c'8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___09():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[1],
        suffix_talea=[8],
        suffix_counts=[1, 2, 3],
        talea_denominator=32,
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'32
                r4
                r16.
                c'4
            }
            {
                c'32
                r16.
                c'4
                c'4
            }
            {
                c'32
                c'4
                c'4
                c'16.
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___10():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r2
                r8
            }
            {
                r2
                r8
            }
            {
                r2
                r8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___11():

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[1],
        suffix_talea=[1],
        suffix_counts=[1],
        talea_denominator=8,
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    c'8
                    r4.
                    c'8
                }
            }
            {
                {
                    c'8
                    r4
                    c'8
                }
            }
            {
                \times 4/7 {
                    c'8
                    r2
                    r8
                    c'8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___12():
    r'''Adds 32nd note to beginning of every division.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[1],
        talea_denominator=32,
        fill_with_notes=False,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[2, 0],
        split_divisions_by_counts=[20],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 8/9 {
                    c'32
                    r2
                    r32
                }
            }
            {
                {
                    c'32
                    r16.
                }
                \tweak text #tuplet-number::calc-fraction-text
                \times 6/7 {
                    c'32
                    r4.
                    r32
                }
            }
            {
                {
                    c'32
                    r8..
                }
                \times 4/5 {
                    c'32
                    r4
                    r32
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___13():
    r'''Incises outer divisions only.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-8],
        prefix_counts=[2],
        suffix_talea=[-3],
        suffix_counts=[4],
        talea_denominator=32,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r4
                r4
                c'8
            }
            {
                c'2 ~
                c'8
            }
            {
                c'4
                r16.
                r16.
                r16.
                r16.
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___14():
    r'''Incises outer divisions only.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[20],
        suffix_talea=[-1],
        suffix_counts=[2],
        talea_denominator=4,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r4
                r4
                r8
            }
            {
                c'2 ~
                c'8
            }
            {
                c'8
                r4
                r4
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___15():
    r'''Unincised notes.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'2 ~
                c'8
            }
            {
                c'2 ~
                c'8
            }
            {
                c'2 ~
                c'8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___16():
    r'''Incises outer divisions only.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[1],
        suffix_talea=[-1],
        suffix_counts=[1],
        talea_denominator=8,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    r8
                    c'2
                }
            }
            {
                {
                    c'2
                }
            }
            {
                \times 4/7 {
                    c'2.
                    r8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___17():
    r'''Incises outer divisions only.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[-1],
        prefix_counts=[1],
        suffix_talea=[-1],
        suffix_counts=[1],
        talea_denominator=8,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 0, 0, 2],
        split_divisions_by_counts=[3, 1, 4, 1, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    r8
                    c'4.
                }
                {
                    c'8
                }
            }
            {
                {
                    c'2
                }
            }
            {
                {
                    c'8
                }
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    c'2
                    r8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___18():
    r'''Incises outer divisions only. Fills with rests.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[8],
        prefix_counts=[2],
        suffix_talea=[3],
        suffix_counts=[4],
        talea_denominator=32,
        fill_with_notes=False,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'4
                c'4
                r8
            }
            {
                r2
                r8
            }
            {
                r4
                c'16. [
                c'16.
                c'16.
                c'16. ]
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___19():
    r'''Incises outer divisions only. Fills with rests.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[20],
        suffix_talea=[1],
        suffix_counts=[2],
        talea_denominator=4,
        fill_with_notes=False,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                c'4
                c'4
                c'8
            }
            {
                r2
                r8
            }
            {
                r8
                c'4
                c'4
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___20():
    r'''Unincised rests.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        fill_with_notes=False,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        )

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/8
                r2
                r8
            }
            {
                r2
                r8
            }
            {
                r2
                r8
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___21():
    r'''Incises outer divisions only. Fills with rests.
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[1],
        suffix_talea=[1],
        suffix_counts=[1],
        talea_denominator=8,
        fill_with_notes=False,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    c'8
                    r2
                }
            }
            {
                {
                    r2
                }
            }
            {
                \times 4/7 {
                    r2.
                    c'8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_IncisedRhythmMaker___call___22():
    r'''Incises outer divisions only. Fills with rests:
    '''

    incise_specifier = rhythmmakertools.InciseSpecifier(
        prefix_talea=[1],
        prefix_counts=[1],
        suffix_talea=[1],
        suffix_counts=[1],
        talea_denominator=8,
        fill_with_notes=False,
        outer_divisions_only=True,
        )

    maker = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=incise_specifier,
        extra_counts_per_division=[1, 0, 0, 0, 2],
        split_divisions_by_counts=[3, 1, 4, 1, 3],
        )

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    c'8
                    r4.
                }
                {
                    r8
                }
            }
            {
                {
                    r2
                }
            }
            {
                {
                    r8
                }
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    r2
                    c'8
                }
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_01():
    r'''Move prolation of full-measure power-of-two tuplet to time signature.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    measure = Measure((2, 8), [tuplet])
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                c'8
                d'8
                e'8
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_02():
    r'''Move prolation of full-measure non-power-of-two tuplet to
    time signature.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(3, 16), [])
    tuplet.extend("c'16 d'16 e'16 f'16 g'16")
    measure = Measure((3, 16), [tuplet])
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 15/80
            \scaleDurations #'(4 . 5) {
                c'32.
                d'32.
                e'32.
                f'32.
                g'32.
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_03():
    r'''Subsume 7:6 tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(6, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8 a'8 b'8")
    measure = Measure((6, 8), [tuplet])
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 21/28
            \scaleDurations #'(4 . 7) {
                c'8.
                d'8.
                e'8.
                f'8.
                g'8.
                a'8.
                b'8.
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_04():
    r'''Subsume tuplet in nonassignable measure.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(5, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8 a'8")
    measure = Measure((5, 8), [tuplet])
    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 15/24
            \scaleDurations #'(2 . 3) {
                c'8 ~
                c'32
                d'8 ~
                d'32
                e'8 ~
                e'32
                f'8 ~
                f'32
                g'8 ~
                g'32
                a'8 ~
                a'32
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_05():
    r'''Subsume nested tuplet.
    '''

    measure = Measure((2, 8), [])
    measure.append(r"\times 2/3 { \times 2/3 { c'16 d'16 e'16 } f'8 g'8 }")

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            \times 2/3 {
                \times 2/3 {
                    c'16
                    d'16
                    e'16
                }
                f'8
                g'8
            }
        }
        '''
        ), format(measure)

    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(
        measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                \times 2/3 {
                    c'16
                    d'16
                    e'16
                }
                f'8
                g'8
            }
        }
        '''
        ), format(measure)

    assert inspect_(measure).is_well_formed()


def test_scoretools_move_full_measure_tuplet_prolation_to_measure_time_signature_06():
    r'''Submsume 6:5. Time signature should go from 5/16 to 15/48.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(5, 16), "c'8 d'8 e'8")
    measure = Measure((5, 16), [tuplet])

    r'''
    {
        \time 5/16
        \tweak text #tuplet-number::calc-fraction-text
        \times 5/6 {
            c'8
            d'8
            e'8
        }
    }
    '''

    scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(measure)

    r'''
    {
        \time 15/48
        \scaleDurations #'(2 . 3) {
            c'8 ~
            c'32
            d'8 ~
            d'32
            e'8 ~
            e'32
        }
    }
    '''

    assert inspect_(measure).is_well_formed()
    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 15/48
            \scaleDurations #'(2 . 3) {
                c'8 ~
                c'32
                d'8 ~
                d'32
                e'8 ~
                e'32
            }
        }
        '''
        ), format(measure)
